@@ -40,12 +40,18 @@
 #include <string.h>
 #include "cpuid.h"
 
+/*
 #ifdef NO_AVX
+#define CPUTYPE_HASWELL CPUTYPE_NEHALEM
+#define CORE_HASWELL CORE_NEHALEM
 #define CPUTYPE_SANDYBRIDGE CPUTYPE_NEHALEM
 #define CORE_SANDYBRIDGE CORE_NEHALEM
 #define CPUTYPE_BULLDOZER CPUTYPE_BARCELONA
 #define CORE_BULLDOZER CORE_BARCELONA
+#define CPUTYPE_PILEDRIVER CPUTYPE_BARCELONA
+#define CORE_PILEDRIVER CORE_BARCELONA
 #endif
+*/
 
 #ifndef CPUIDEMU
 
@@ -53,9 +59,16 @@
 void cpuid(int op, int *eax, int *ebx, int *ecx, int *edx);
 #else
 static inline void cpuid(int op, int *eax, int *ebx, int *ecx, int *edx){
+#if defined(__i386__) && defined(__PIC__)
+  __asm__ __volatile__
+    ("mov %%ebx, %%edi;"
+     "cpuid;"
+     "xchgl %%ebx, %%edi;"
+     : "=a" (*eax), "=D" (*ebx), "=c" (*ecx), "=d" (*edx) : "a" (op) : "cc");
+#else
   __asm__ __volatile__
     ("cpuid": "=a" (*eax), "=b" (*ebx), "=c" (*ecx), "=d" (*edx) : "a" (op) : "cc");
-
+#endif
 }
 #endif
 
@@ -87,7 +100,7 @@ void cpuid(unsigned int op, unsigned int *eax, unsigned int *ebx, unsigned int *
   if ((current < start) || (current > stop)) current = start;
 
   while ((count > 0) && (idlist[current].id != op)) {
-    
+
     current ++;
     if (current > stop) current = start;
     count --;
@@ -128,9 +141,9 @@ int support_avx(){
 #ifndef NO_AVX
   int eax, ebx, ecx, edx;
   int ret=0;
-  
+
   cpuid(1, &eax, &ebx, &ecx, &edx);
-  if ((ecx & (1 << 28)) != 0 && (ecx & (1 << 27)) != 0){
+  if ((ecx & (1 << 28)) != 0 && (ecx & (1 << 27)) != 0 && (ecx & (1 << 26)) != 0){
     xgetbv(0, &eax, &edx);
     if((eax & 6) == 6){
       ret=1;  //OS support AVX
@@ -148,7 +161,7 @@ int get_vendor(void){
   char vendor[13];
 
   cpuid(0, &eax, &ebx, &ecx, &edx);
-  
+
   *(int *)(&vendor[0]) = ebx;
   *(int *)(&vendor[4]) = edx;
   *(int *)(&vendor[8]) = ecx;
@@ -169,7 +182,7 @@ int get_vendor(void){
 
   return VENDOR_UNKNOWN;
 }
-  
+
 int get_cputype(int gettype){
   int eax, ebx, ecx, edx;
   int extend_family, family;
@@ -178,7 +191,7 @@ int get_cputype(int gettype){
   int feature = 0;
 
   cpuid(1, &eax, &ebx, &ecx, &edx);
-  
+
   switch (gettype) {
   case GET_EXFAMILY :
     return BITMASK(eax, 20, 0xff);
@@ -225,6 +238,7 @@ int get_cputype(int gettype){
     if ((ecx & (1 << 20)) != 0) feature |= HAVE_SSE4_2;
 #ifndef NO_AVX
     if (support_avx()) feature |= HAVE_AVX;
+    if ((ecx & (1 << 12)) != 0) feature |= HAVE_FMA3;
 #endif
 
     if (have_excpuid() >= 0x01) {
@@ -247,12 +261,12 @@ int get_cputype(int gettype){
   }
   return feature;
 }
-  
+
 int get_cacheinfo(int type, cache_info_t *cacheinfo){
   int eax, ebx, ecx, edx, cpuid_level;
   int info[15];
   int i;
-  cache_info_t LC1, LD1, L2, L3, 
+  cache_info_t LC1, LD1, L2, L3,
     ITB, DTB, LITB, LDTB,
     L2ITB, L2DTB, L2LITB, L2LDTB;
 
@@ -278,22 +292,22 @@ int get_cacheinfo(int type, cache_info_t *cacheinfo){
     info[ 0] = BITMASK(eax,  8, 0xff);
     info[ 1] = BITMASK(eax, 16, 0xff);
     info[ 2] = BITMASK(eax, 24, 0xff);
-    
+
     info[ 3] = BITMASK(ebx,  0, 0xff);
     info[ 4] = BITMASK(ebx,  8, 0xff);
     info[ 5] = BITMASK(ebx, 16, 0xff);
     info[ 6] = BITMASK(ebx, 24, 0xff);
-    
+
     info[ 7] = BITMASK(ecx,  0, 0xff);
     info[ 8] = BITMASK(ecx,  8, 0xff);
     info[ 9] = BITMASK(ecx, 16, 0xff);
     info[10] = BITMASK(ecx, 24, 0xff);
-    
+
     info[11] = BITMASK(edx,  0, 0xff);
     info[12] = BITMASK(edx,  8, 0xff);
     info[13] = BITMASK(edx, 16, 0xff);
     info[14] = BITMASK(edx, 24, 0xff);
-    
+
     for (i = 0; i < 15; i++){
 
       switch (info[i]){
@@ -859,7 +873,7 @@ int get_cacheinfo(int type, cache_info_t *cacheinfo){
     LITB.associative = BITMASK(eax,  8, 0xff);
     if (LITB.associative == 0xff) LITB.associative = 0;
     LITB.linesize    = BITMASK(eax,  0, 0xff);
-    
+
     DTB.size        = 4;
     DTB.associative = BITMASK(ebx, 24, 0xff);
     if (DTB.associative == 0xff) DTB.associative = 0;
@@ -891,7 +905,7 @@ int get_cacheinfo(int type, cache_info_t *cacheinfo){
     L2LITB.associative = BITMASK(eax,  8, 0xff);
     if (L2LITB.associative == 0xff) L2LITB.associative = 0;
     L2LITB.linesize    = BITMASK(eax,  0, 0xff);
-    
+
     L2DTB.size        = 4;
     L2DTB.associative = BITMASK(ebx, 24, 0xff);
     if (L2DTB.associative == 0xff) L2DTB.associative = 0;
@@ -915,7 +929,7 @@ int get_cacheinfo(int type, cache_info_t *cacheinfo){
   }
 
     switch (type) {
-      
+
     case CACHE_INFO_L1_I :
       *cacheinfo = LC1;
       break;
@@ -977,7 +991,7 @@ int get_cpuname(void){
       return CPUTYPE_PENTIUM;
     case 0x6:
       switch (exmodel) {
-      case 0: 
+      case 0:
 	switch (model) {
 	case  1:
 	case  3:
@@ -1017,8 +1031,8 @@ int get_cpuname(void){
       case  2:
 	switch (model) {
 	case 5:
-	  //Intel Core (Clarkdale) / Core (Arrandale) 
-	  // Pentium (Clarkdale) / Pentium Mobile (Arrandale) 
+	  //Intel Core (Clarkdale) / Core (Arrandale)
+	  // Pentium (Clarkdale) / Pentium Mobile (Arrandale)
 	  // Xeon (Clarkdale), 32nm
 	  return CPUTYPE_NEHALEM;
 	case 10:
@@ -1046,12 +1060,38 @@ int get_cpuname(void){
       case 3:
 	switch (model) {
 	case 10:
+        case 14:
+	  // Ivy Bridge
 	  if(support_avx())
 	    return CPUTYPE_SANDYBRIDGE;
 	  else
 	    return CPUTYPE_NEHALEM;
+        case 12:
+	case 15:
+          if(support_avx())
+#ifndef NO_AVX2
+            return CPUTYPE_HASWELL;
+#else
+	    return CPUTYPE_SANDYBRIDGE;
+#endif
+          else
+	    return CPUTYPE_NEHALEM;
 	}
 	break;
+      case 4:
+        switch (model) {
+        case 5:
+	case 6:
+          if(support_avx())
+#ifndef NO_AVX2
+            return CPUTYPE_HASWELL;
+#else
+	    return CPUTYPE_SANDYBRIDGE;
+#endif
+          else
+	    return CPUTYPE_NEHALEM;
+        }
+        break;
       }
       break;
     case 0x7:
@@ -1084,11 +1124,21 @@ int get_cpuname(void){
       case  1:
       case 10:
 	return CPUTYPE_BARCELONA;
-      case  6:   //AMD Bulldozer Opteron 6200 / Opteron 4200 / AMD FX-Series
-	if(support_avx())
-	  return CPUTYPE_BULLDOZER;
-	else
-	  return CPUTYPE_BARCELONA; //OS don't support AVX.
+      case  6:
+	switch (model) {
+	case 1:
+	  //AMD Bulldozer Opteron 6200 / Opteron 4200 / AMD FX-Series
+	  if(support_avx())
+	    return CPUTYPE_BULLDOZER;
+	  else
+	    return CPUTYPE_BARCELONA; //OS don't support AVX.
+	case 2:
+	  if(support_avx())
+	    return CPUTYPE_PILEDRIVER;
+	  else
+	    return CPUTYPE_BARCELONA; //OS don't support AVX.
+	}
+	break;
       case  5:
 	return CPUTYPE_BOBCAT;
       }
@@ -1213,6 +1263,8 @@ static char *cpuname[] = {
   "SANDYBRIDGE",
   "BOBCAT",
   "BULLDOZER",
+  "PILEDRIVER",
+  "HASWELL",
 };
 
 static char *lowercpuname[] = {
@@ -1262,11 +1314,13 @@ static char *lowercpuname[] = {
   "sandybridge",
   "bobcat",
   "bulldozer",
+  "piledriver",
+  "haswell",
 };
 
 static char *corename[] = {
   "UNKOWN",
-  "80486", 
+  "80486",
   "P5",
   "P6",
   "KATMAI",
@@ -1288,11 +1342,13 @@ static char *corename[] = {
   "SANDYBRIDGE",
   "BOBCAT",
   "BULLDOZER",
+  "PILEDRIVER",
+  "HASWELL",
 };
 
 static char *corename_lower[] = {
   "unknown",
-  "80486", 
+  "80486",
   "p5",
   "p6",
   "katmai",
@@ -1314,6 +1370,8 @@ static char *corename_lower[] = {
   "sandybridge",
   "bobcat",
   "bulldozer",
+  "piledriver",
+  "haswell",
 };
 
 
@@ -1391,8 +1449,8 @@ int get_coretype(void){
       case  2:
 	switch (model) {
 	case 5:
-	  //Intel Core (Clarkdale) / Core (Arrandale) 
-	  // Pentium (Clarkdale) / Pentium Mobile (Arrandale) 
+	  //Intel Core (Clarkdale) / Core (Arrandale)
+	  // Pentium (Clarkdale) / Pentium Mobile (Arrandale)
 	  // Xeon (Clarkdale), 32nm
 	  return CORE_NEHALEM;
 	case 10:
@@ -1420,12 +1478,37 @@ int get_coretype(void){
       case 3:
 	switch (model) {
 	case 10:
+	case 14:
 	  if(support_avx())
 	    return CORE_SANDYBRIDGE;
 	  else
 	    return CORE_NEHALEM; //OS doesn't support AVX
+        case 12:
+	case 15:
+          if(support_avx())
+#ifndef NO_AVX2
+            return CORE_HASWELL;
+#else
+	    return CORE_SANDYBRIDGE;
+#endif
+          else
+	    return CORE_NEHALEM;
 	}
 	break;
+      case 4:
+        switch (model) {
+        case 5:
+	case 6:
+          if(support_avx())
+#ifndef NO_AVX2
+            return CORE_HASWELL;
+#else
+	    return CORE_SANDYBRIDGE;
+#endif
+          else
+	    return CORE_NEHALEM;
+        }
+        break;
       }
       break;
 
@@ -1439,14 +1522,22 @@ int get_coretype(void){
     if (family <= 0x5) return CORE_80486;
     if (family <= 0xe) return CORE_ATHLON;
     if (family == 0xf){
-      if ((exfamily == 0) || (exfamily == 2)) return CORE_OPTERON; 
-      else if (exfamily == 5) return CORE_BOBCAT; 
+      if ((exfamily == 0) || (exfamily == 2)) return CORE_OPTERON;
+      else if (exfamily == 5) return CORE_BOBCAT;
       else if (exfamily == 6) {
-	//AMD Bulldozer Opteron 6200 / Opteron 4200 / AMD FX-Series
-	if(support_avx())
-	  return CORE_BULLDOZER;
-	else
-	  return CORE_BARCELONA; //OS don't support AVX. Use old kernels.
+	switch (model) {
+	case 1:
+	  //AMD Bulldozer Opteron 6200 / Opteron 4200 / AMD FX-Series
+	  if(support_avx())
+	    return CORE_BULLDOZER;
+	  else
+	    return CORE_BARCELONA; //OS don't support AVX.
+	case 2:
+	  if(support_avx())
+	    return CORE_PILEDRIVER;
+	  else
+	    return CORE_BARCELONA; //OS don't support AVX.
+	}
       }else return CORE_BARCELONA;
     }
   }
@@ -1479,35 +1570,41 @@ void get_cpuconfig(void){
       printf("#define L1_CODE_ASSOCIATIVE %d\n", info.associative);
       printf("#define L1_CODE_LINESIZE %d\n", info.linesize);
     }
-    
+
     get_cacheinfo(CACHE_INFO_L1_D, &info);
     if (info.size > 0) {
       printf("#define L1_DATA_SIZE %d\n", info.size * 1024);
       printf("#define L1_DATA_ASSOCIATIVE %d\n", info.associative);
       printf("#define L1_DATA_LINESIZE %d\n", info.linesize);
     }
-    
+
     get_cacheinfo(CACHE_INFO_L2, &info);
     if (info.size > 0) {
       printf("#define L2_SIZE %d\n", info.size * 1024);
       printf("#define L2_ASSOCIATIVE %d\n", info.associative);
       printf("#define L2_LINESIZE %d\n", info.linesize);
+    } else {
+      //fall back for some virtual machines.
+      printf("#define L2_SIZE 1048576\n");
+      printf("#define L2_ASSOCIATIVE 6\n");
+      printf("#define L2_LINESIZE 64\n");
     }
-    
+
+
     get_cacheinfo(CACHE_INFO_L3, &info);
     if (info.size > 0) {
       printf("#define L3_SIZE %d\n", info.size * 1024);
       printf("#define L3_ASSOCIATIVE %d\n", info.associative);
       printf("#define L3_LINESIZE %d\n", info.linesize);
     }
-    
+
     get_cacheinfo(CACHE_INFO_L1_ITB, &info);
     if (info.size > 0) {
       printf("#define ITB_SIZE %d\n", info.size * 1024);
       printf("#define ITB_ASSOCIATIVE %d\n", info.associative);
       printf("#define ITB_ENTRIES %d\n", info.linesize);
     }
-    
+
     get_cacheinfo(CACHE_INFO_L1_DTB, &info);
     if (info.size > 0) {
       printf("#define DTB_SIZE %d\n", info.size * 1024);
@@ -1517,7 +1614,7 @@ void get_cpuconfig(void){
       //fall back for some virtual machines.
       printf("#define DTB_DEFAULT_ENTRIES 32\n");
     }
-    
+
     features = get_cputype(GET_FEATURE);
 
     if (features & HAVE_CMOV )   printf("#define HAVE_CMOV\n");
@@ -1534,12 +1631,13 @@ void get_cpuconfig(void){
     if (features & HAVE_3DNOWEX) printf("#define HAVE_3DNOWEX\n");
     if (features & HAVE_3DNOW)   printf("#define HAVE_3DNOW\n");
     if (features & HAVE_FMA4 )    printf("#define HAVE_FMA4\n");
+    if (features & HAVE_FMA3 )    printf("#define HAVE_FMA3\n");
     if (features & HAVE_CFLUSH)  printf("#define HAVE_CFLUSH\n");
     if (features & HAVE_HIT)     printf("#define HAVE_HIT 1\n");
     if (features & HAVE_MISALIGNSSE) printf("#define HAVE_MISALIGNSSE\n");
     if (features & HAVE_128BITFPU)   printf("#define HAVE_128BITFPU\n");
     if (features & HAVE_FASTMOVU)    printf("#define HAVE_FASTMOVU\n");
-    
+
     printf("#define NUM_SHAREDCACHE %d\n", get_cputype(GET_NUMSHARE) + 1);
     printf("#define NUM_CORES %d\n", get_cputype(GET_NUMCORES) + 1);
 
@@ -1601,5 +1699,6 @@ void get_sse(void){
   if (features & HAVE_3DNOWEX) printf("HAVE_3DNOWEX=1\n");
   if (features & HAVE_3DNOW)   printf("HAVE_3DNOW=1\n");
   if (features & HAVE_FMA4 )    printf("HAVE_FMA4=1\n");
+  if (features & HAVE_FMA3 )    printf("HAVE_FMA3=1\n");
 
 }
